#define _GNU_SOURCE

#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netinet/ip.h>
#include <dlfcn.h>
#include <stdlib.h>

int
socket(int domain, int type, int protocol)
{
  static int(*realsocket)(int, int, int);
  static int tos = -1;

  if (!realsocket) {
    realsocket = dlsym(RTLD_NEXT, "socket");
    if (!realsocket)
      abort();
    const char *v = getenv("LIBSETTOS_TOS");
    if (v) {
      char *ptr;
      int ttos = strtol(v, &ptr, 0);
      if (*ptr == 0 && !(ttos & 3) && ttos >= 0 && ttos < 256) {
	tos = ttos;
      }
    }
  }
  int sock = realsocket(domain, type, protocol);
  if (sock >= 0)
    switch (domain)
      {
      case PF_INET:
	setsockopt(sock, SOL_IP, IP_TOS, &tos, sizeof(tos));
	break;
      case PF_INET6:
	setsockopt(sock, SOL_IPV6, IPV6_TCLASS, &tos, sizeof(tos));
	break;
      default:
	break;
      }

  return sock;
}
